# -*- coding: utf-8 -*-
from django.contrib import admin
from django import forms
from django import forms
from basket.models import Order
from django.utils.translation import ugettext_lazy as _


class CommentForm(forms.ModelForm):
    class Meta:
        model = Order

class OrderAdmin(admin.ModelAdmin):
    model = Order
    form = CommentForm
    exclude = ['user', 'session_key', ]
    list_display = ['__unicode__', 'goods', 'summary', 'status', 'registered', 'user']
    list_filter = ['status']
    fieldsets = (
        (_('Order data'), {
            'classes': ('collapse',),
            'fields': ('status', 'created', 'comment'),
        }),
    )
    search_fields = ('user__username',)


try:
    admin.site.register(Order, OrderAdmin)
except admin.sites.AlreadyRegistered:
    pass



# -*- coding: utf-8 -*-
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

PRICE_ATTR = getattr(settings, 'BASKET_PRICE_ATTR', 'price')
DEFAULT_BASKET_FORM = 'basket.forms.DefaultOrderForm'

BASKET_FORM = getattr(settings, 'BASKET_FORM', DEFAULT_BASKET_FORM)

BASKET_OPTIONS_USE_KEEP = getattr(settings, 'BASKET_OPTIONS_USE_KEEP', True)
BASKET_OPTIONS_USE_DELETE = getattr(settings, 'BASKET_OPTIONS_USE_DELETE', False)

STATUS_PENDING = 0
STATUS_NEW = 1
STATUS_PROCESS = 2
STATUS_CLOSED = 3
STATUS_ERROR = 4

DEFAULT_ORDER_STATUSES = (
    (STATUS_PENDING, _('Pending')),
    (STATUS_NEW, _('New')),
    (STATUS_PROCESS, _('Process')),
    (STATUS_CLOSED, _('Closed & OK')),
    (STATUS_ERROR, _('Closed with error')),
)
ORDER_STATUSES = getattr(settings, 'ORDER_STATUSES', DEFAULT_ORDER_STATUSES)

DEFAULT_ORDER_EMAIL_SUBJECT = 'New order from site'
ORDER_EMAIL_SUBJECT = getattr(settings, 'ORDER_EMAIL_SUBJECT', DEFAULT_ORDER_EMAIL_SUBJECT)



# -*- coding: utf-8 -*-
from basket.models import Order
from basket.settings import BASKET_OPTIONS_USE_DELETE
from django.core.exceptions import ObjectDoesNotExist
from django.utils.safestring import mark_safe
from django.core.urlresolvers import reverse
import re

_BASKET_JS_RE = \
    re.compile(r'(<script\W[^>]*\bsrc\s*=\s*(\'|"|)[^\'^\"]*basket.js(\'|"|)\b[^>]*>\W*</\s*script\s*>)', re.IGNORECASE)
BASKET_URL_SCRIPT = '''
<script type="text/javascript">
basket.add_url = '%(add)s';
basket.del_url = '%(del)s';
</script>
'''

class BasketMiddleware(object):
    '''Add order attribute to request only if appropriate Order instance exists'''
    def process_request(self, request):
        if request.user.is_authenticated():
            if request.user.order_set.active_orders().count() == 1:
                request.order = request.user.order_set.active_orders().get()
                request.session['order_id'] = request.order.id
            elif request.user.order_set.active_orders().count() == 0:
                request.order = None
            else:
                # exception situation, two orders created!
                request.order = request.user.order_set.active_orders()[0]
                request.session['order_id'] = request.order.id
        else:
            if 'order_id' in request.session:
                try:
                    request.order = Order.objects.active_orders().get(
                        id=request.session['order_id'])
                except ObjectDoesNotExist:
                    # Order completed
                    del request.session['order_id']
                    request.order = None
            else:
                request.order = None

    def process_response(self, request, response):
        '''Add basket url to script'''
        urls = {
            'add': reverse('add_to_basket'),
        }
        if BASKET_OPTIONS_USE_DELETE:
            urls['del'] = reverse('del_from_basket')
        else:
            urls['del'] = ''
        try:
            def add_url_definition(match):
                return mark_safe(match.group() + BASKET_URL_SCRIPT % urls)
            response.content = _BASKET_JS_RE.sub(add_url_definition, response.content)
        except:
            return response
        else:
            return response



import os
import django.core.mail
from django.conf import settings
from django.shortcuts import render_to_response
from django.template import RequestContext
from basket.settings import BASKET_FORM

def send_mail(subject, message, recipient_list):
    from_email = settings.DEFAULT_FROM_EMAIL
    if type(recipient_list) is not list:
        recipient_list = [recipient_list, ]
    try:
        django.core.mail.send_mail(subject, message, from_email, recipient_list)
    except Exception, e:
        print 'Error while sending mail: ', e

def render_to(template_path):
    def decorator(func):
        def wrapper(request, *args, **kwargs):
            output = func(request, *args, **kwargs)
            if output is None:
                output = {}
            if not isinstance(output, dict):
                return output
            return render_to_response(template_path, output,
                context_instance=RequestContext(request))
        return wrapper
    return decorator

def import_item(path, error_text=''):
    """Imports a model by given string. In error case raises ImpoprelyConfigured"""
    i = path.rfind('.')
    module, attr = path[:i], path[i + 1:]
    try:
        return getattr(__import__(module, {}, {}, ['']), attr)
    except ImportError, e:
        from django.core.exceptions import ImproperlyConfigured
        raise ImproperlyConfigured('Error importing %s %s: "%s"' % (error_text, path, e))

def get_order_form():
    return import_item(BASKET_FORM)



# -*- coding: utf-8 -*-
from django import forms
from django.forms.models import inlineformset_factory
from django.contrib.contenttypes.models import ContentType
from basket.models import Order, BasketItem
from basket.settings import BASKET_OPTIONS_USE_KEEP
from django.utils.translation import ugettext_lazy as _


class BasketItemForm(forms.ModelForm):
    class Meta:
        model = BasketItem

    content_type = forms.ModelChoiceField(queryset=ContentType.objects.all(),
        widget=forms.HiddenInput)
    object_id = forms.IntegerField(widget=forms.HiddenInput)
    if BASKET_OPTIONS_USE_KEEP:
        keep = forms.BooleanField(initial=True, required=False)

    def save(self, *args, **kwargs):
        if BASKET_OPTIONS_USE_KEEP:
            if not self.cleaned_data.get('keep', False):
                self.cleaned_data['quantity'] = 0
        self.instance.order.set_quantity(self.instance.content_object,
            self.cleaned_data.get('quantity', 0))

OrderFormset = inlineformset_factory(Order, BasketItem, extra=0,
    can_delete=False, form=BasketItemForm)


class DefaultOrderForm(forms.Form):
    name = forms.CharField(label=_('Customer name'), max_length=100)
    phone = forms.CharField(label=_('Customer phone'), max_length=100)
    address = forms.CharField(label=_('Delivery address'), max_length=255)
    contact_time = forms.CharField(label=_('Convenient time to call'),
        max_length=50, required=False)
    comment = forms.CharField(label=_('Comment for us'), max_length=255,
        widget=forms.Textarea(), required=False)

    def __init__(self, request, *args, **kwds):
        return super(DefaultOrderForm, self).__init__(*args, **kwds)



# -*- coding: utf-8 -*-
from django.conf.urls.defaults import *
from basket.settings import BASKET_OPTIONS_USE_DELETE

urlpatterns = patterns('basket.views',
    url(r'^$', 'basket', name='basket'),
    url(r'^confirm/$', 'confirm', name='order_confirm'),
    url(r'^add/$', 'add_to_basket', name='add_to_basket'),
)

if BASKET_OPTIONS_USE_DELETE:
    urlpatterns += patterns('basket.views',
        url(r'^delete/$', 'delete_from_basket', name='del_from_basket'),
    )

urlpatterns += patterns('django.views.generic.simple',
    url(r'^thankyou/$', 'direct_to_template', {'template': 'basket/thankyou.html'},
        name='basket-thankyou'),
    url(r'^empty/$', 'direct_to_template', {'template': 'basket/empty.html'},
        name='basket-empty'),
)
